#!/usr/bin/env python
"""
###############################################################################
# FILE NAME : server.py
# AUTHOR : J.Enochs
# CREATION DATE : 07-Sep-2016
# LAST MODIFIED : 29-Aug-2019
# DESCRIPTION : This server performs various functions depending on what day
#               the python course is in.
#
###############################################################################/
"""

import os
import re
import time
import shelve
import random
import socket
import hashlib
import operator 
import threading
import select
from subprocess import call
from subprocess import Popen, PIPE
from uuid import getnode as get_mac
from threading import Thread, Semaphore
from netifaces import interfaces, ifaddresses, AF_INET

THREAD_LOCK = threading.Semaphore(value=1)

# CONFIGURATION ########################################
# WHEN USING OPENSTACK:  Place network file containing IP on top line in same directory as server.py
#                        Also change INTERFACE_MODE below to 'e' when using OpenStack

PORTS = {5555:"Flag"}
TRANSFER_REGISTRATION = None  # Put student's name here in lowercase and in quotes
OPEN_REGISTRATION = True  # Names must be in roster.txt
ALLOW_RAW_FLAGS = True
INTERFACE_MODE = 'w'  # Use 'w' when using WiFi and 'e' when using wired connection or openstack

# END CONFIGURATION ########################################


CAPTUREDFLAGS = {}
MAC2SCORES = {} 
SERVERMAC = None
MAC2NAME = {}
SVRIP = None
FLAGS = {}
MULTI_FLAGS = {}
ROSTER = []
REGISTERED = []

def main():
    os.system("clear")
    try:
        load_core_data()
        Server()
    except (KeyboardInterrupt, SystemExit):
        pass
    except (socket.timeout): 
	print("\nSocket timed out.\n")

def load_core_data():
    global CAPTUREDFLAGS
    global REGISTERED
    global MAC2SCORES
    global SERVERMAC
    global MAC2NAME
    global SVRIP
    global FLAGS
    global MULTI_FLAGS
    global ROSTER
    
    # Load class roster
    if os.path.isfile("roster.txt"):
        try:
            with open("roster.txt", "r") as roster_file:
                for line in roster_file:
                    ROSTER.append(line.lower().strip())
        except Exception as e:
            print(e)
            print("\nFailed to load class roster\n")
            exit(0)

    # Get the student to MAC dictionary and registered list
    try:
        db = shelve.open("../Quiz_Server/initial_quiz_scores.db")
        if "macDict" in db:
            MAC2NAME = db["macDict"]
        else:
            print("\nMAC DATA FAILED TO LOAD\n")
        if "registered" in db:
            REGISTERED = db["registered"]
    except Exception as e:
        print(e)
        print("\nMAC DATA FAILED TO LOAD\n")
        exit(0)
    finally:
        db.close()

    try:
        SERVERMAC = get_mac() 
    except:
        print("\nFAILED TO READ SERVER MAC ADDRESS!\n")
        exit(0)

    # Get server IP
    try:
        for ifaceName in interfaces():
            if ifaceName[0] == INTERFACE_MODE:
                try:
                    SVRIP = ifaddresses(ifaceName)[AF_INET][0]['addr']
                except:
                    set_IP()
                try:
                    SVRIP = ifaddresses(ifaceName)[AF_INET][0]['addr']
                except:
                    raise
        # OpenStack Hack
        if SVRIP == "10.0.0.1" or SVRIP == "10.0.0.2":
            filePath = "network"
            if os.path.isfile(filePath):
                with open(filePath) as network_file:
                    SVRIP = network_file.readline().strip()
            else:
                print("\nNo network file found! Using IP {}".format(SVRIP))

    except Exception as e:
        print(e)
        print("Check network interface.  IP could not be configured.")
        exit(0)

            
    # Get the current CTF scores for the class
    try:
        db2 = shelve.open("../class_flag_scores.db")
        if "mac2scores" in db2:
            MAC2SCORES = db2["mac2scores"]
        else:
            print("\nSCORES FAILED TO LOAD\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nSCORES FAILED TO LOAD\n")
        exit(0)
    finally:
        db2.close()

    # Get captured flags
    try:
        db = shelve.open("captured_flags.db")
        if "capturedFlags" in db:
            CAPTUREDFLAGS = db["capturedFlags"]
        else:
            print("\nCAPTURED FLAGS FAILED TO LOAD\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCAPTURED FLAGS FAILED TO LOAD\n")
        exit(0)
    finally:
        db.close()
    
    # Get the database of flags
    try:
        db = shelve.open("flags.db")
        if "flagsDict" in db:
            FLAGS = db["flagsDict"] 
            FLAGS["top10"] = "top10"
            FLAGS["score"] = "score"
            FLAGS["scores"] = "scores"
            FLAGS["report"] = "report"
        else:
            print("\nFLAGS FAILED TO LOAD\n")
        if "multiDict" in db:
            MULTI_FLAGS = db["multiDict"] 
        else:
            print("\nMULTI FLAGS FAILED TO LOAD\n")
    except Exception as e:
        print(e)
        print("\nFLAGS FAILED TO LOAD\n")
        exit(0)
    finally:
        db.close()

def set_IP():
    defaultIPfound = False
    addresses = {}
    for ifaceName in interfaces():
        singleAddresses = [i['addr'] for i in ifaddresses(ifaceName).setdefault(AF_INET, [{'addr':'No IP addr'}] )]
        print("{} = {}".format(ifaceName, singleAddresses[0]))
        addresses[ifaceName] = singleAddresses[0]
        for key in addresses:
            if addresses[key] == '192.168.15.9':
                defaultIPfound = True
            elif addresses[key] == 'No IP addr':
                if ifaceName[0] != 'w':
                    call(["sudo", "ifconfig", ifaceName, "192.168.15.9","netmask", "255.255.255.0", "up"])
                    print("Interface {} set to 192.168.159".format(ifaceName))
                    defaultIPfound = True
                    singleAddresses = [i['addr'] for i in ifaddresses(ifaceName).setdefault(AF_INET, [{'addr':'No IP addr'}] )]
                    addresses[ifaceName] = singleAddresses[0]
    if not defaultIPfound:
        print("\n192.168.15.9 not set")
        for key in addresses:
            print("   {} = {}".format(key, addresses[key]))
    else:
        print("\nUsing local IP 192.168.15.9")
        for key in addresses:
            print("   {} = {}".format(key, addresses[key]))
    print("\nServer started...")

class Server(object):
    """Threaded server"""

    # static variable
    sockets = []

    def __init__(self):
        for port in PORTS:
            adr = ("0.0.0.0", port)
            Address = adr
            self.s = socket.socket()
            self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.s.bind(Address)
            self.s.listen(3)
            Server.sockets.append(self.s)
            self.last_port = port
            print("\nRunning {} server on port {}".format(PORTS[port], port))
        if OPEN_REGISTRATION:
            print("\nREGISTRATION IS OPEN!\n\n")
        if TRANSFER_REGISTRATION != None:
            print("\nREGISTRATION OPEN FOR {}\n\n".format(TRANSFER_REGISTRATION))
        #self.generate_class_report()
        #exit(0)
            
        while True:
            # Wait for any of the sockets to get a connection
            readable,_,_ = select.select(Server.sockets, [], [])
            ready_socket = readable[0]
            client, address = ready_socket.accept()
            client.settimeout(1)
            Thread(target = self.listenToClient, args = (client, address)).start()

    def listenToClient(self, client, address):
        self.Client = client
        self.Adr = address
        flag = client.recv(1024)
        name, mac, score = self.get_personal_data(address, flag) 
        print("\nName: {} Score: {}".format(name, score))
        skt = client.getsockname()
        if skt[1] == 5555:
            pass
            #print("Port = {}".format(skt[1]))
        elif skt[1] == 8081:
            print("Port = {}".format(skt[1]))
            #time.sleep(90)
            #client.send("\n     HOLD!\n\n     HOLD!\n\n     HOLD!\n\n")
            flag = "86bdd7dd325aa95c5589e873ac600090077bf6"
        elif skt[1] == 1916:
            print("Port = {}".format(skt[1]))
            time.sleep(90)
            #flag = "a34fc894c082b7bef96bb034a6609c1342f2c9"
        elif skt[1] == 1970:
            print("Port = {}".format(skt[1]))
            time.sleep(90)
            #flag = "d1508f56e3c155aec4923b599d9f7b1be01533"
        elif skt[1] == 1990:
            print("Port = {}".format(skt[1]))
            time.sleep(90)
            #flag = "d1508f56e3c155aec4923b599d9f7b1be01530"
        elif skt[1] == 9090:
            pass
        else:
            print("\n\n  *** Unexpected Port ***\n")
        if name == "NOTFOUND":
            msg = "\n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n               YOUR COMPUTER IS NOT REGISTERED  \n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n\n".format(name.upper())
        else:
            msg = self.check_Flag(flag, mac, name) 
        print(msg)
        client.send(msg)
        client.close()

    def generate_class_report(self):
        names = [MAC2NAME[k] for k in MAC2NAME]
        score = 0
        for mac in MAC2NAME:
            score = MAC2SCORES[mac]
            name = MAC2NAME[mac]
            self.generate_reportcard(name)
            rptcard = self.generate_reportcard(name)
            sorted_rptcard = sorted(rptcard.items(), key=operator.itemgetter(1))
            msg = "\n     <<<<<<<<<<<<<<<<<<<< SCORE {} >>>>>>>>>>>>>>>>>>>>\n".format(score)
            for item in sorted_rptcard:
                #msg = msg + "     " + item + "\n"
                msg = msg + "     " + str(item[0]) + "\n"
            msg = msg + "     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n\n"
            print(msg)
            with open("class_rpt.txt", "a") as rpt:
                rpt.write("%s" % msg)
    def generate_reportcard(self, name):
        reportcard = {}
        score = 0
        flag = None
        exer = None
        pts = 0 
        for key in CAPTUREDFLAGS:
            if CAPTUREDFLAGS[key] == name:
                if len(key) > 42: 
                    flag = self.unmake_unique_flag(key)
                    if flag in FLAGS:
                        exer = FLAGS[flag]
                        if exer in MULTI_FLAGS:
                            pts = MULTI_FLAGS[exer]
                        else:
                            pts = 100 
                        if "NICE try" in exer:
                            continue
                    else:
                        exer = "Unknown: not found"
                        continue
                    newkey = name + "_" + exer
                    if exer not in reportcard.values():
                        if "-TryGit" not in exer:
                            if "-Active" not in exer:
                                reportcard[newkey] = exer
                else:
                    exer = "Unknown: small"
                    reportcard[key] = exer
            elif name.lower() in key.lower():
                if "NICE try" in key:
                    continue
                exer = CAPTUREDFLAGS[key]
                pts = 0 
                if exer in MULTI_FLAGS:
                    pts = MULTI_FLAGS[exer]
                else:
                    pts = 100 
                if exer not in reportcard.values():
                    if "-TryGit" not in exer:
                        if "-Active" not in exer:
                            reportcard[key] = exer
        return reportcard

    def record_registration(self):
        global REGISTERED
        global MAC2NAME
        global MAC2SCORES
        try:
            THREAD_LOCK.acquire()
            db = shelve.open("../Quiz_Server/initial_quiz_scores.db")
            if "registered" in db:
                db["registered"] = REGISTERED
            else:
                print("FAILED TO RECORD REGISTRATION")
            if "macDict" in db:
                db["macDict"] = MAC2NAME
            else:
                print("FAILED TO RECORD MAC TO NAME")
        except:
            print("\nFAILED TO OPEN CAPTURED FLAG DATABASE!")
        finally:
            db.close()
            THREAD_LOCK.release()
        try:
            THREAD_LOCK.acquire()
            db2 = shelve.open("../class_flag_scores.db")
            if "mac2scores" in db2:
                db2["mac2scores"] = MAC2SCORES
            else:
                print("FAILED TO RECORD SCORE!")
                print("INFORM INSTRUCTOR")
        finally:
            db2.close()
            THREAD_LOCK.release()


    def record_results(self, name, exer, flag):
        global CAPTUREDFLAGS
        global MAC2SCORES
        key = name + "_" + exer
        CAPTUREDFLAGS[key] = exer
        if len(flag) > 42:
            key = flag
            CAPTUREDFLAGS[key] = name
        try:
            THREAD_LOCK.acquire()
            db = shelve.open("captured_flags.db")
            if "capturedFlags" in db:
                db["capturedFlags"] = CAPTUREDFLAGS
            else:
                print("FAILED TO RECORD CAPTURED FLAG!")
                print("INFORM INSTRUCTOR")
        except:
            print("\nFAILED TO OPEN CAPTURED FLAG DATABASE!")
        finally:
            db.close()
            THREAD_LOCK.release()
        try:
            THREAD_LOCK.acquire()
            db2 = shelve.open("../class_flag_scores.db")
            if "mac2scores" in db2:
                db2["mac2scores"] = MAC2SCORES
            else:
                print("FAILED TO RECORD SCORE!")
                print("INFORM INSTRUCTOR")
        finally:
            db2.close()
            THREAD_LOCK.release()

    def unmake_unique_flag(self, flag):

        # Count leading zeros

        # Remove 4 digit code
        code4 = flag[0:2] + flag[-2:]
        stripped_flag = flag[2:-2]

        # Modified flag from hex string back to long int
        long_flag = int(stripped_flag, 16)
        # Reverse XOR using magic 4
        sha2 = hashlib.sha1()
        sha2.update(code4)
        anti_flag = sha2.hexdigest()
        anti_flag = int(anti_flag, 16)
        flag_decimal = long_flag ^ anti_flag
        # Back to hex as the original flag
        original_flag = hex(flag_decimal)[2:-1]
        length = len(original_flag)
        if length < 40:
            num = 40 - length
            for _ in range(num):
                original_flag = "0" + original_flag

        return original_flag

    def check_Flag(self, flag, mac, name):
        global CAPTUREDFLAGS
        global TRANSFER_REGISTRATION
        exer = None
        coin = 100
        info_only = False
        flag = flag.strip()
        flag_original = flag.strip()
        if len(flag_original) > 41:
            flag = self.unmake_unique_flag(flag_original)
        elif len(flag_original) > 38:  # so I know if a standard 40 get's in.
            msg = "     Invalid flag, wrong length."
            return msg 
        try:
            flag_lowered = flag.lower().strip() 
            if ALLOW_RAW_FLAGS:
                # if the flag does not appear in our databases...
                if flag_lowered not in FLAGS and flag_lowered not in ROSTER:
                    # sha1 hash it...
                    flag_hashed = hashlib.sha1(flag_lowered).hexdigest()
                    # and check again... 
                    if flag_hashed.lower().strip() in FLAGS:
                        # if we now find a record of the hash, make that the flag
                        # these 4 lines allow submission of unhashed flags
                        flag_lowered = flag_hashed.lower().strip()
            if flag_lowered in FLAGS or flag_lowered in ROSTER:
                if flag_lowered in ROSTER:
                    exer = flag_lowered
                    flag = flag_lowered
                    if mac in MAC2NAME:
                        name = MAC2NAME[mac]
                    else:
                        name = "Unknown"
                    if mac in MAC2SCORES:
                        score = MAC2SCORES[mac]
                    else:
                        score = 0
                    if not OPEN_REGISTRATION:
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n               REGISTRATION CLOSED\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n"
                    else:
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n               REGISTRATION ERROR\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n"
                exer = FLAGS[flag_lowered]
                if exer in MULTI_FLAGS:
                    coin = MULTI_FLAGS[exer]
                if exer in FLAGS and len(flag) > 38: 
                    student_list = FLAGS[exer]
                    students = []
                    for student in student_list:
                        student = str(student).lower()
                        students.append(student)
                    name_lower = str(name).lower()
                    if name_lower not in students:
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n          NO RECORD OF YOUR COMPLETION\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n"
                        return msg
                if name is not None:
                    captured_key = name + "_" + exer
                    captured_key2 = flag_original
                if captured_key in CAPTUREDFLAGS:
                    if "Ex" in flag:
                        info_only = True
                    else:
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n          YOU ALREADY CAPTURED THIS FLAG\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n"
                        return msg
                elif captured_key2 in CAPTUREDFLAGS:
                    name = CAPTUREDFLAGS[captured_key2]
                    msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n         FLAG ALREADY USED BY: {} \n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(name)
                    return msg
                if "Ex" in flag:
                    if info_only == False:
                        if mac in MAC2SCORES:
                            score = MAC2SCORES[mac]
                            score = score - 1
                            MAC2SCORES[mac] = score
                        else:
                            score = 0
                            MAC2SCORES[mac] = score
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n     -1pt: {}\n\n     {}\n\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(score, exer)
                    else:
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n     HINT: Relook\n\n     {}\n\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(exer)
                elif "score" == flag.lower():
                    info_only = True
                    if mac in MAC2SCORES:
                        score = MAC2SCORES[mac]
                    else:
                        score = 0
                    msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n     NAME: {}   SCORE: {}   \n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(name.upper(), score)
                elif "report" == flag.lower():
                    if mac in MAC2SCORES:
                        score = MAC2SCORES[mac]
                    else:
                        score = 0
                    info_only = True
                    rptcard = self.generate_reportcard(name)
                    sorted_rptcard = sorted(rptcard.items(), key=operator.itemgetter(1))
                    msg = "\n     <<<<<<<<<<<<<<<<<<<< SCORE {} >>>>>>>>>>>>>>>>>>>>\n".format(score)
                    for item in sorted_rptcard:
                        msg = msg + "     " + str(item[0]) + "\n"
                    msg = msg + "     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n\n"
                elif "top10" == flag.lower():
                    info_only = True
                    keys = list(MAC2SCORES)
                    length = len(keys)
                    name2scores = {}
                    for key in keys:
                        score = MAC2SCORES[key]
                        name = MAC2NAME[key]
                        name2scores[name] = score
                    if length < 10:
                        for x in range(length + 1,11):
                            name2scores["< " + str(x) + " >"] = x * -.1
                    key_list = sorted(name2scores, key=name2scores.__getitem__)
                    sorted_x = sorted(name2scores.items(), key=operator.itemgetter(1), reverse=True)
                    cut2ten = sorted_x[0:10]

                    msg = "\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\t\t{} {}\n\n".format(cut2ten[0][0].upper(), cut2ten[0][1], cut2ten[1][0].upper(), cut2ten[1][1], cut2ten[2][0].upper(), cut2ten[2][1], cut2ten[3][0].upper(), cut2ten[3][1], cut2ten[4][0].upper(), cut2ten[4][1], cut2ten[5][0].upper(), cut2ten[5][1], cut2ten[6][0].upper(), cut2ten[6][1], cut2ten[7][0].upper(), cut2ten[7][1], cut2ten[8][0].upper(), cut2ten[8][1], cut2ten[9][0].upper(), cut2ten[9][1])
                elif "NICE" in exer:
                    if mac in MAC2SCORES:
                        pass
                        score = MAC2SCORES[mac]
                        #score = score + 1
                        #MAC2SCORES[mac] = score
                    else:
                        score = 0
                        MAC2SCORES[mac] = score
                    msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n     {}\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(exer) 
                elif flag_lowered in ROSTER:
                    info_only = True
                    if mac in MAC2SCORES and flag_lowered != TRANSFER_REGISTRATION:
                        score = MAC2SCORES[mac]
                        msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n     NAME: {}   SCORE: {}   \n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(name.upper(), score)
                    else:
                        try:
                            if flag_lowered == TRANSFER_REGISTRATION:
                                old_mac =  MAC2NAME.keys()[MAC2NAME.values().index(flag.lower())]
                                if old_mac:
                                    #print("\nOld mac is {}\n".format(old_mac))
                                    #print("\nNew mac is {}\n".format(mac))
                                    current_score = 0
                                    if old_mac in MAC2NAME:
                                        #print("Deleting old_mac from MAC2NAME")
                                        del MAC2NAME[old_mac]
                                    MAC2NAME[mac] = flag_lowered
                                    if old_mac in MAC2SCORES:
                                        current_score = MAC2SCORES[old_mac]
                                        #print("Current score is {}\n".format(current_score))
                                        #print("Deleting old_mac from MAC2SCORES")
                                        del MAC2SCORES[old_mac]
                                    MAC2SCORES[mac] = current_score
                                    TRANSFER_REGISTRATION = None
                                    msg = "\n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n       NAME: {}   TRANSFER SUCCESSFUL   SCORE: {}   \n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n\n".format(flag.upper(), current_score)
                                else:
                                    msg = "\nMAC lookup failed!\n"
                            elif OPEN_REGISTRATION:
                                if flag_lowered not in REGISTERED:
                                    if mac not in MAC2NAME:
                                        MAC2NAME[mac] = flag.lower()
                                    if mac not in MAC2SCORES:
                                        MAC2SCORES[mac] = 0
                                        msg = "\n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n     NAME: {}   REGISTRATION SUCCESSFUL   SCORE: {}   \n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n\n".format(flag.upper(), score)
                                        REGISTERED.append(flag_lowered)
                                else:
                                    msg = "\n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n              NAME: {} ALREADY REGISTERED  \n     <<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>\n\n".format(name.upper())
                        except Exception as e:
                            print(e)
                        self.record_registration()
                else:
                    THREAD_LOCK.acquire()
                    if mac in MAC2SCORES:
                        score = MAC2SCORES[mac]
                        #print(adv)
                        score = score + coin
                        MAC2SCORES[mac] = score
                    else:
                        score = coin
                        MAC2SCORES[mac] = score
                    THREAD_LOCK.release()
                    msg = "\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n     {}   +{}pts: {}\n     <<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>\n\n".format(exer, coin, score)
            else:
                info_only = True
                msg = "     Invalid Flag: not in database."
        except Exception as e:
            info_only == True
            print(e)
            msg = "     Invalid flag, error."
        if info_only == False:
            self.record_results(name, exer, flag_original)
        return msg

    def get_personal_data(self, address, flag):
        name = None
        mac = None
        try:
            #print("\nSVRIP: {}, self.Adr: {}\n".format(SVRIP, self.Adr))
            if SVRIP not in self.Adr:
                try: 
                    p = Popen(['ping', self.Adr[0], '-c1'], stdout=PIPE, stderr=PIPE)
                    out, err = p.communicate()
                    p = Popen(["arp", "-n", self.Adr[0]], stdout=PIPE)
                    s = p.communicate()[0]
                    mac = re.search(r"(([a-f\d]{1,2}\:){5}[a-f\d]{1,2})", s).groups()[0]
                    #mac = self.Adr[0] #this uses IP instead of mac
                except Exception as e:
                    print(e)
                    print("FAILED TO DETECT MAC ADDRESS 1")
                    exit(0)
            else:
                mac = SERVERMAC
        except Exception as e:
            print(e)
            print("FAILED TO DETECT MAC ADDRESS 2")
            exit(0)
        if isinstance(mac, basestring):
            if len(mac) < 10:
                print("Invalid MAC address")
                exit(0)
        try:
            if mac in MAC2NAME:
                name = MAC2NAME[mac]
            else:
                if flag.lower().strip() in ROSTER: 
                    name = flag.lower().strip()
                else:
                    print("\nFAILED TO FIND NAME IN DATABASE\n")
                    name = "NOTFOUND"
        except Exception as e:
            print("FAILED TO FIND YOUR NAME IN DATABASE {}".format(e))
            exit(0)

        if mac in MAC2SCORES:
            score = MAC2SCORES[mac]
        else: 
            score = 0
            
        #print("\nName: {}, mac: {}, score: {}\n".format(name, mac, score))
        return name, mac, score


if __name__ == "__main__":
    main()
